/*
 * Copyright (C) 2015, Wazuh Inc.
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation.
 */

#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>
#include <cmocka.h>
#include <stdio.h>

#include "../../wazuh_modules/wmodules.h"
#include "wazuh_modules/vulnerability_detector/wm_vuln_detector.h"

static int setup_run_now(void ** state) {
    *state = calloc(1, sizeof(wm_vuldet_t));
    return 0;
}

static int teardown_run_now(void ** state) {
    free(*state);
    return 0;
}

void test_wm_vuldet_set_run_now(void ** state) {
    wm_vuldet_t * vuldet = *(wm_vuldet_t **)state;
    wm_vuldet_set_run_now(vuldet);

    assert_true(wm_vuldet_check_run_now(vuldet));
}

void test_wm_vuldet_clear_run_now(void ** state) {
    wm_vuldet_t * vuldet = *(wm_vuldet_t **)state;
    wm_vuldet_set_run_now(vuldet);
    wm_vuldet_clear_run_now(vuldet);

    assert_false(wm_vuldet_check_run_now(vuldet));
}

void test_wm_vuldet_query_run_now(void ** state) {
    wm_vuldet_t * vuldet = *(wm_vuldet_t **)state;
    char input[] = "run_now";
    char * output = NULL;
    const char EXPECTED_OUTPUT[] = "ok {\"error\":0,\"message\":\"The scan has been requested\"}";

    vuldet->flags.enabled = 1;

    size_t n = wm_vuldet_query(vuldet, input, &output);

    assert_string_equal(output, EXPECTED_OUTPUT);
    assert_int_equal(n, strlen(EXPECTED_OUTPUT));
    assert_true(wm_vuldet_check_run_now(vuldet));

    free(output);
}

void test_wm_vuldet_query_run_now_again(void ** state) {
    wm_vuldet_t * vuldet = *(wm_vuldet_t **)state;
    char input[] = "run_now";
    char * output = NULL;
    const char EXPECTED_OUTPUT[] = "err {\"error\":10,\"message\":\"A scan has already been requested\"}";

    vuldet->flags.enabled = 1;

    wm_vuldet_set_run_now(vuldet);
    size_t n = wm_vuldet_query(vuldet, input, &output);

    assert_string_equal(output, EXPECTED_OUTPUT);
    assert_int_equal(n, strlen(EXPECTED_OUTPUT));
    assert_true(wm_vuldet_check_run_now(vuldet));

    free(output);
}

void test_wm_vuldet_query_run_now_disabled(void ** state) {
    wm_vuldet_t * vuldet = *(wm_vuldet_t **)state;
    char input[] = "run_now";
    char * output = NULL;
    const char EXPECTED_OUTPUT[] = "err {\"error\":11,\"message\":\"This module is disabled\"}";

    wm_vuldet_set_run_now(vuldet);
    size_t n = wm_vuldet_query(vuldet, input, &output);

    assert_string_equal(output, EXPECTED_OUTPUT);
    assert_int_equal(n, strlen(EXPECTED_OUTPUT));
    assert_true(wm_vuldet_check_run_now(vuldet));

    free(output);
}

void test_wm_vuldet_query_unknown(void ** state) {
    wm_vuldet_t * vuldet = *(wm_vuldet_t **)state;
    char input[] = "none";
    char * output = NULL;
    const char EXPECTED_OUTPUT[] = "err {\"error\":12,\"message\":\"Unknown command\"}";

    size_t n = wm_vuldet_query(vuldet, input, &output);

    assert_string_equal(output, EXPECTED_OUTPUT);
    assert_int_equal(n, strlen(EXPECTED_OUTPUT));
    assert_false(wm_vuldet_check_run_now(vuldet));

    free(output);
}

int main() {
    const struct CMUnitTest tests[] = {
        // Test wm_vuldet_*_run_now
        cmocka_unit_test_setup_teardown(test_wm_vuldet_set_run_now, setup_run_now, teardown_run_now),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_clear_run_now, setup_run_now, teardown_run_now),

        // Test wm_vuldet_query
        cmocka_unit_test_setup_teardown(test_wm_vuldet_query_run_now, setup_run_now, teardown_run_now),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_query_run_now_again, setup_run_now, teardown_run_now),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_query_run_now_disabled, setup_run_now, teardown_run_now),
        cmocka_unit_test_setup_teardown(test_wm_vuldet_query_unknown, setup_run_now, teardown_run_now),
    };

    return cmocka_run_group_tests(tests, NULL, NULL);
}
